/*
 * Copyright 2022 Omico
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package me.omico.gradm

import me.omico.gradm.internal.YamlDocument
import me.omico.gradm.internal.maven.collectAllMetadata
import me.omico.gradm.internal.sha1
import java.nio.file.Path
import kotlin.io.path.exists
import kotlin.io.path.readLines
import kotlin.io.path.writeText

typealias VersionsMeta = Map<String, String>

val YamlDocument.localVersionsMeta: VersionsMeta
    get() = runCatching(::collectAllMetadata)
        .getOrDefault(emptyList())
        .associate { it.module to it.latestVersion }

fun VersionsMeta.store(versionsMeta: Path, versionsMetaHash: Path) {
    versionsMeta.writeText(versionsMetaContent)
    versionsMetaHash.writeText(versionsMeta.sha1())
}

fun Path.asVersionsMeta(): VersionsMeta? =
    takeIf(Path::exists)?.readLines()
        ?.associate {
            val strings = it.split("=")
            strings.first() to strings.last()
        }

fun Path.asVersionsMetaHash(): String? = takeIf(Path::exists)?.readLines()?.firstOrNull()

private val VersionsMeta.versionsMetaContent: String
    get() = buildString { toSortedMap().forEach { (key, value) -> appendLine("$key=$value") } }
